/*
    This file is part of Reversi DS.

    Reversi DS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Reversi DS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Reversi DS.  If not, see <http://www.gnu.org/licenses/>.
     
    http://ReversiDS.sourceforge.net
*/
#include "Board.h"

int Board::valid(int x,int y, int color)const{
	int success=0;
	if(b[x][y]!=E)
		return 0;
	success+=validLine(x+1,y  , 1, 0,color);
	success+=validLine(x-1,y  ,-1, 0,color);
	success+=validLine(x  ,y+1, 0, 1,color);
	success+=validLine(x  ,y-1, 0,-1,color);
	success+=validLine(x+1,y+1, 1, 1,color);
	success+=validLine(x+1,y-1, 1,-1,color);
	success+=validLine(x-1,y+1,-1, 1,color);
	success+=validLine(x-1,y-1,-1,-1,color);
	return success;		
}
///returns positive if the line starting at (x,y) is vunerable to 
///play, ei if by playing a piece at (x-dx, y-dy ) of piece color
/// will turn over pieces (x+n*dx,y+n*dy) for n>0
int Board::validLine(int x,int y,int dx,int dy,int color)const{
	if(x<0 || x>=8 || y<0 || y>=8) // off board
		return 0;
	if(b[x][y]==E)
		return 0;
	if(b[x][y]==color)
		return 0;
	x+=dx;
	y+=dy;
	while(1){
		if(x<0 || x>=8 || y<0 || y>=8)
			return 0;
		if(b[x][y]==E)
			return 0;
		if(b[x][y]==color)
			return 1;
		x+=dx;
		y+=dy;
	}
	return 0;			
}
int Board::play(int x,int y,int color){
	if(b[x][y]!=E)
		return 0;
	b[x][y]=color;
	int success=0;
	success+=updateLine(x+1,y  , 1, 0,color);
	success+=updateLine(x-1,y  ,-1, 0,color);
	success+=updateLine(x  ,y+1, 0, 1,color);
	success+=updateLine(x  ,y-1, 0,-1,color);
	success+=updateLine(x+1,y+1, 1, 1,color);
	success+=updateLine(x+1,y-1, 1,-1,color);
	success+=updateLine(x-1,y+1,-1, 1,color);
	success+=updateLine(x-1,y-1,-1,-1,color);
	if(success)
		return 1;
	b[x][y]=E;
	return 0;
}
int Board::updateLine(int x,int y,int dx,int dy,int color){
	//check to see if line is to be flipped
	while(1){
		if(x<0 || x>=8 || y<0 || y>=8)//out of bounds
			return 0;
		if(b[x][y]==E)//empty cell
			return 0;
		if(b[x][y]==color)//end of line found
			break;
		x+=dx;
		y+=dy;				
	}
	x-=dx;
	y-=dy;
	if(b[x][y]==color)//neighboring tile was same color as new tile
		return 0;
	while(b[x][y]!=color){
		b[x][y]=color;
		x-=dx;
		y-=dy;
	}
	return 1;		
}

