/*
    This file is part of Reversi DS.

    Reversi DS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Reversi DS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Reversi DS.  If not, see <http://www.gnu.org/licenses/>.
     
    http://ReversiDS.sourceforge.net
*/
#pragma once
#include "Gfx.h"

class Characters{
public:
	Characters(const void* p, int row, int col, int tileW, int tileH ){
		Gfx raw(p,row,col);
		charW = tileW;
		charH = tileH;
		charSize = charW*charH;
		
		data = new u16[row*col];
		int c_r = 3;
		int c_w = 27;
		int offset = 0;
		for( int ty=0; ty<c_r; ++ty ){
			for( int tx=0; tx<c_w; ++tx){
				for( int y=0;y<tileH;++y){
					for(int x=0; x<tileW;++x){
						data[offset++]=raw.data[ (ty*tileH+y)*256+tx*tileW+x ];
					}
				}
			}
		}
		lower = data;
		upper = data + 27*charSize;	
		number = upper + 27*charSize;	
	}
	~Characters(){
		delete [] data;
	}
	u16* operator()(char c){
		if( c >='A' && c<='Z' )
			return upper+(c-'A')*charSize;
		else if( c >= 'a' && c<= 'z' )
			return lower+(c-'a')*charSize;
		else if( c >= '0' && c <= '9' )
			return number+(c-'0')*charSize;
		else{
			switch( c ){
			case ',':
				return number+10*charSize;
				break;
			case '.':
				return number+11*charSize;
				break;
			case '?':
				return number+12*charSize;
				break;
			case '!':
				return number+13*charSize;
				break;
			case '*':
				return number+14*charSize;
				break;
			case '-':
				return number+15*charSize;
				break;
			case '\'':
				return number+16*charSize;
				break;
			}
		}
			
		return 0;
		
	}	
	int charSize;
	int charH;
	int charW;
	///owned data
	///not in human readable format, charaters are flattened
	///so that their data is continuously stored one row after another
	u16* data;
	///pointers, don't delete directly
	u16* lower;
	u16* upper;	
	u16* number;
};
