/*
    This file is part of Reversi DS.

    Reversi DS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Reversi DS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Reversi DS.  If not, see <http://www.gnu.org/licenses/>.
     
    http://ReversiDS.sourceforge.net
*/
#include "OthelloData.h"
#include <iostream>
using namespace std;
OthelloData::OthelloData(const Board& b,int turn,const OthelloMove& lm)
:b(b),sig(b),turn(turn),freeMoves(0){
	for(int x=0;x<8;x++){
		for(int y=0;y<8;y++){
			if( b.valid(x,y,turn) )
				freeMoves++;
		}
	}
		
	lastMove=lm;
}
OthelloData::~OthelloData(){

}
int OthelloData::evaluate()const{
	if(ended()){
		int whiteCount=0;
		int blackCount=0;

		for(int x=0;x<8;x++){
			for(int y=0;y<8;y++){
				if(b(x,y)==b.W)
					whiteCount++;
				if(b(x,y)==b.B)
					blackCount++;
			}
		}
		if(whiteCount>blackCount)
			return 9999;
		else 
			return -9999;
	}
	int W[8][8];
	int NW[8][8];
	int N[8][8];
	int NE[8][8];
	int v;
	for(int y=0;y<8;y++){
		for(int x=0;x<8;x++){
			v=b(x,y);
			if(v==b.E){
				W[x][y]=EMPTY;
				NW[x][y]=EMPTY;
				N[x][y]=EMPTY;
				NE[x][y]=EMPTY;
				continue;
			}
			int &w = W[x][y];
			w=evalPiece(x,y,1,0);
			if(w==PREV)
				w=W[x-1][y];
			
			int &nw = NW[x][y];
			nw=evalPiece(x,y,1,1);
			if(nw==PREV)
				nw=NW[x-1][y-1];
			
			int &n = N[x][y];
			n=evalPiece(x,y,0,1);
			if(n==PREV)
				n=N[x][y-1];
			
			int &ne = NE[x][y];
			ne=evalPiece(x,y,-1,1);
			if(ne==PREV)
				ne=NE[x+1][y-1];
		}
	}
	int score[8][8];
	for( int x=0;x<8;x++){
		for( int y=0; y<8;y++){
			int &w = W[x][y];
			int &nw = NW[x][y];
			int &n = N[x][y];
			int &ne = NE[x][y];
			if( w <= nw && w <= n && w <= ne )
				score[x][y] = w;
			else if( nw <= n && nw <= ne )
				score[x][y] = nw;
			else if( n <= ne )
				score[x][y] = n;
			else
				score[x][y] = ne;
		}
	}
	int whites=0;
	int blacks=0;
	for(int x=0;x<8;x++){
		for(int y=0;y<8;y++){
			
			if(b(x,y)==b.W)
				whites+=score[x][y];
			else if(b(x,y)==b.B)
				blacks+=score[x][y];
		}
	}
	return whites-blacks;
}

int OthelloData::evalPiece(int x,int y,int sx,int sy)const{
	
	if(x-sx<0 || y-sy<0 || x-sx==8)
		return SAFE;
	else{
		const int v=b(x,y);
		int px = x-sx;
		int py = y-sy;
		for(;; px-=sx, py-=sy ){
			if( px < 0 || py < 0 || px == 8 || py == 8 )
				return SAFE;
			if( b( px, py ) != v )
				break;			
		}
		int tx=x;
		int ty=y;
		while(1){
			tx+=sx;
			ty+=sy;
			if(tx==8 || ty==8 || tx<0)
				return SAFE;
			if(v==b(tx,ty))
				continue;
			if( b(tx,ty) == b(px, py) )
				return PROTECTED;
			else
				return PLACED;
		}
	}	
}



int OthelloData::ended()const{
	Board temp=b;
	for(int x=0;x<8;x++){
		for(int y=0;y<8;y++){
			if(temp.play(x,y,b.W))
				return 0;
			if(temp.play(x,y,b.B))
				return 0;
		}
	}
	return 1;
}
vector<OthelloData> OthelloData::enumerateOptions()const{
	vector<OthelloData> options;
	options.reserve(5);
	Board temp=b;
	int nextTurn;
	if(turn==b.B)
		nextTurn=b.W;
	else
		nextTurn=b.B;
	for( int x=0;x<8;x++){
		for(int y=0;y<8;y++){
			if(temp.play(x,y,turn)){
				options.push_back(OthelloData(temp,nextTurn,OthelloMove(x,y,turn)));
				temp=b;
			}
		}
	}
	if(options.empty()){
		options.push_back(OthelloData(temp,nextTurn,OthelloMove()));
	}
	return options;
}
