#pragma once
#include "Board.h"


class Signature{
	public:
	int stamp;
	Signature(int stamp): stamp(stamp){};
};

class QuadSig{
public:
	int diag;
	int left;
	int right;
	/*
	 * M[i][j]
	 * 0--j
	 * |\
	 * | \
	 * i
	 */
	void set(int q[4][4]){
		diag=0;
		left=0;
		right=0;
		for(int x=0;x<4;x++){
			for(int y=0;y<4;y++){
				if(q[x][y]==Board::E)
					q[x][y]=0;
				else if(q[x][y]==Board::B)
					q[x][y]=1;
				else 
					q[x][y]=2;
			}
		}
		diag=(q[0][0]) + (q[1][1]<<2) + (q[2][2]<<4) + (q[3][3]<<6);
		right =(q[1][0])+(q[2][0]<<2)+(q[3][0]<<4)+(q[1][2]<<6)+(q[1][3]<<8)+
				(q[2][3]<<10);
		left=(q[0][1])+(q[0][2]<<2)+(q[0][3]<<4)+(q[2][1]<<6)+(q[3][1]<<8)+
				(q[3][2]<<10);
		
	}
	void mirror(){
		int temp=left;
		left=right;
		right=temp;
	}
	//assumes 32 bit int
	unsigned int score() const{
		return (diag<<24)+(left<<12)+right;
	}
	unsigned int order() const{
		return diag + left + right;
	}
	bool operator==(const QuadSig &that)const{
		return diag==that.diag && left==that.left && right==that.right;
	}
};
/**
 * Want a signature that will identify all permutations of the board
 * 	so we want rotations, mirrors, and rotations + mirrors
 * 	to have the same signature 
 */
class BoardSig: public Signature{
public:
	QuadSig A,B,C,D;
	BoardSig():Signature(-1){}
	BoardSig(const Board &b,int stamp=0):Signature(stamp){
		/*
		 * 0---- +j
		 * |a b
		 * |d c
		 * +i
		 * 
		 * for each, need
		 *  |
		 * -0-
		 *  |#
		 */
		QuadSig quad[4];
		int qA[4][4];
		int qB[4][4];
		int qC[4][4];
		int qD[4][4];
		
		
		for(int i=0;i<4;i++){
			for(int j=0;j<4;j++){
				qA[3-i][3-j]= b( i  , j   );
				qB[  j][3-i]= b( i  , j+4 );
				qC[  i][  j]= b( i+4, j+4 );
				qD[3-j][  i]= b( i+4, j   );
			}
		}
		quad[0].set( qA );
		quad[1].set( qB );
		quad[2].set( qC );
		quad[3].set( qD );
		
		int s[4];
		int max1 =0;
		int max2 =0;
		int mi =0;
		/* need to order the quads
		 * 		find the largest order() quad, set to A
		 * 		if the largest neighbor is counter-clockwise, mirror the board
		 */ 
		for(int x=0;x<4;x++)
			s[x]=quad[x].order();
		
		max1 = s[0];
		max2 = max1 - s[2];
		
		for(int x = 1; x<4; x++){
			if( s[x] > max1 ){
				mi = x;
				max1 = s[x];
				max2 = max1 - s[(x+2)%4];
			} else if( s[x] == max1 ){
				if( s[x] - s[(x+2)%4] > max2 ){
					mi = x;
					max1=s[x];
					max2=s[x]-s[(x+2)%4];
				}
			}
		}
		int mi_plus = mi + 1;
		if( mi_plus > 3 )
			mi_plus = 0;
		int mi_minus = mi - 1;
		if( mi_minus < 0 )
			mi_minus = 3;
		int mi_opp = mi + 2;
		if( mi_opp > 3 )
			mi_opp -= 4;
		else if( mi_opp < 0 )
			mi_opp += 4;
		
		if( s[ mi_plus ] >= s[ mi_minus ] ){
			A = quad[mi];
			B = quad[ mi_plus ];
			C = quad[ mi_opp ];
			D = quad[ mi_minus ];	
		} else {
			for( int x=0; x<4; ++x)
				quad[x].mirror();
			A = quad[mi];
			B = quad[ mi_minus ];  
			C = quad[ mi_opp ];	
			D = quad[ mi_plus ];
		}		
		
	}
	unsigned int getHash(int max){
		//simplistic hash...might need to change, but hash table gets completely filled anyways
		return (A.score() ^ B.score() ^ C.score() ^ D.score())%max;	
	}
	bool equals(const BoardSig &s)const{
		//BoardSig *bs = (BoardSig *)&s;
		//return A==bs->A && B==bs->B && C==bs->C && D==bs->D;
		return A==s.A && B==s.B && C==s.C && D==s.D;
	} 
		
		
};
