/*
    This file is part of Reversi DS.

    Reversi DS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Reversi DS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Reversi DS.  If not, see <http://www.gnu.org/licenses/>.
     
    http://ReversiDS.sourceforge.net
*/

// need to figure some way of keeping track of sprites
// only 128 total available...
// [0,64] are the game board pieces
// 65 is the image for placing the piece
// 67 is the opponent
//

#include <nds.h>

#include "Sprite.h"
#include "Board.h"
#include "Game.h"
#include "Maxmin.h"
#include "OthelloData.h"
#include "Gfx.h"
#include "Characters.h"
#include <iostream>
#include <stdio.h>
#include <sstream>

using namespace std;

//pictures
#include "title.h"
#include "board256.h"
#include "pieces.h"
#include "character.h"
#include "letters.h"
#include "messageBox.h"

const int BOARD_START_X=42; 
const int BOARD_START_Y=8;
const int BOARD_STEP=22;


//Graphics
//////////////////////////////////////////////
Sprite opponent[4];
Characters letters(lettersBitmap,256,48,8,16);
Sprite black;
Sprite white;
Sprite gray;
Gfx box(messageBoxBitmap,150,150);

// background and forground uids
// TODO: hide away
int fore;
int back;

int lock=0; // mutex...need to implement in assembly
bool message = false;

// index for the computer avatar state
// TODO: encapsulate the computer avatar
int INDEX=0;

void timer(){
	if(INDEX)
		INDEX=0;
	else
		INDEX++;
	oamSet(&oamSub,67,96,128,0,15,opponent[0].size,opponent[0].format,opponent[INDEX].data,0,false,false,false,false,false);
	oamUpdate(&oamSub);
}
void toggleMenu(){
	if( lock++ == 0 ){ 	
		scanKeys();
		if( keysDown() & KEY_START ){
			if( message )
				videoBgDisable(fore);
			else
				videoBgEnable(fore);
			message = !message;
		} 
	}	
	--lock;			
}

void initSprites(Sprite &sp,int index){
	u16* data=new u16[128*32];
	decompress(piecesBitmap,data, LZ77Vram);
	sp.data=oamAllocateGfx(&oamMain, sp.size, sp.format);
	for(int y=0;y<32;y++){
		for(int x=0;x<32;x++){
			sp.data[y*32+x]=data[y*128+x+index*32];
		}
	}
	delete [] data;
}
void initCharacter(Sprite *sp){
	u16* data=new u16[64*256];
	decompress(characterBitmap,data,LZ77Vram);
	sp[0].data=oamAllocateGfx(&oamSub,sp[0].size,sp[0].format);
	sp[1].data=oamAllocateGfx(&oamSub,sp[1].size,sp[1].format);
	sp[2].data=oamAllocateGfx(&oamSub,sp[2].size,sp[2].format);
	sp[3].data=oamAllocateGfx(&oamSub,sp[3].size,sp[3].format);
	for(int y=0;y<64;y++){
		for(int x=0;x<64;x++){
			sp[0].data[y*64+x]=data[y*64+x];
			sp[1].data[y*64+x]=data[y*64+x+64*64];
			sp[2].data[y*64+x]=data[y*64+x+64*64*2];
			sp[3].data[y*64+x]=data[y*64+x+64*64*3];
		}
	}
	delete [] data;
}
///clears global sprites
void clearSprites(){
	for(int x =0; x<4; ++x)
		oamFreeGfx(&oamSub,opponent[x].data);
	oamFreeGfx(&oamMain,black.data);
	oamFreeGfx(&oamMain,white.data);
	oamFreeGfx(&oamMain,gray.data);	
	
}
// handles sprites 0 through 64 inclusive
// ie the pieces of the game board and the next piece to be played
void drawBoard(Game &b){
	int spriteCount=1;
	if(b.turn==b.b.B)
		oamSet(&oamMain,0,5,5,1,15,black.size,black.format,black.data,0,false,false,false,false,false);
	else
		oamSet(&oamMain,0,5,5,1,15,white.size,white.format,white.data,0,false,false,false,false,false);
	for(int x=0;x<8;x++){
		for(int y=0;y<8;y++){
			if(b.b(x,y)==b.b.B){
				oamSet(&oamMain,spriteCount,BOARD_START_X+x*BOARD_STEP,BOARD_START_Y+y*BOARD_STEP,1,15,
						black.size,black.format,black.data,0,false,false,false,false,false);
				spriteCount++;
			} else if(b.b(x,y)==b.b.W){
				oamSet(&oamMain,spriteCount,BOARD_START_X+x*BOARD_STEP,BOARD_START_Y+y*BOARD_STEP,1,15,
						white.size,white.format,white.data,0,false,false,false,false,false);
				spriteCount++;
			}
		}
	}
}
void constructMenu(u16* bg,Gfx& box,Characters& letters,const string& message){
	static const int SDIM = 256;
	static int X_OFF = SDIM/2 - box.cols/2;
	static const int Y_OFF = 20;
	for( int y=0; y<box.rows; ++y ){
		const int bgRow = (y+Y_OFF)*SDIM;
		for( int x=0; x<box.cols; ++x ){
			bg[bgRow+x+X_OFF]=box.data[y*box.cols+x];
			
		}
	}
	X_OFF += 4;
	int X = X_OFF;
	int Y = Y_OFF+5;
	for( string::const_iterator i=message.begin(); i!=message.end(); ++i ){
		switch(*i){
			case '\n':
				Y += letters.charH;
				X = X_OFF;
				continue;
				break;
			case ' ':
				X+= letters.charW;
				continue;
				break;
			default:
			{
				u16* p = letters(*i);
				for(int y = 0; y<letters.charH; ++y ){
					for(int x = 0; x<letters.charW; ++x ){
						if( *p >> 15 )
							bg[ (Y+y)*256 + (X+x) ] = *p;
						++p;				
					}
				}
				X += letters.charW;
				break;
			}
		}
		
	}	
}
void constructInfoMenu(){
	stringstream ss;
	ss <<"    Reversi DS    "<<endl
	   <<"------------------"<<endl
	   <<"   developed by   "<<endl
	   <<"    Darren Earl   "<<endl
	   <<" ReversiDS.sf.net "<<endl
	   <<"  using devKitPro "<<endl
	   <<"  licensed under  "<<endl
	   <<"      GPL v3      "<<endl
	   <<" 	 press start    ";
	string s = ss.str();
	u16* p = (u16*)bgGetMapPtr(fore);
	constructMenu(p,box,letters,s);
}

void initialize(){	
	///power on hardware	
	powerOn(POWER_ALL_2D);
	lcdMainOnBottom();
	
	vramSetMainBanks(VRAM_A_MAIN_BG,
					 VRAM_B_MAIN_BG,
					 VRAM_C_SUB_BG,
					 VRAM_D_SUB_SPRITE);
	vramSetBankE(VRAM_E_MAIN_SPRITE);
	
	videoSetMode(MODE_5_2D);	
	videoSetModeSub(MODE_5_2D);
	
	videoBgDisable(0); // text mode unused
	videoBgDisable(1); // text mode unused
	videoBgEnable(2);
	videoBgEnable(3);
	
	back = bgInit(3, BgType_Bmp16, BgSize_B16_256x256, 8,0);
	fore = bgInit(2, BgType_Bmp16, BgSize_B16_256x256, 0,0);
	
	bgInitSub(3, BgType_Bmp16, BgSize_B16_256x256, 0,0);
	
	///Load Gfx, visable entire game
	decompress(titleBitmap, BG_GFX_SUB, LZ77Vram);
	decompress(board256Bitmap, bgGetGfxPtr(back),  LZ77Vram);
	
	
	oamInit(&oamMain,SpriteMapping_Bmp_1D_128,false);
	oamInit(&oamSub,SpriteMapping_Bmp_1D_128,false);
	
	opponent[0].size=SpriteSize_64x64;
	opponent[0].format=SpriteColorFormat_Bmp;
	
	opponent[1].size=SpriteSize_64x64;
	opponent[1].format=SpriteColorFormat_Bmp;
	
	opponent[2].size=SpriteSize_64x64;
	opponent[2].format=SpriteColorFormat_Bmp;
	
	opponent[3].size=SpriteSize_64x64;
	opponent[3].format=SpriteColorFormat_Bmp;
	initCharacter(opponent);
	
	black.size=SpriteSize_32x32;
    black.format=SpriteColorFormat_Bmp;
	white.size=SpriteSize_32x32;
	white.format=SpriteColorFormat_Bmp;
	gray.size=SpriteSize_32x32;
	gray.format=SpriteColorFormat_Bmp;
	initSprites(black,1);
	initSprites(white,0);
	initSprites(gray,2);

	///start timer for updating screen
	irqSet(IRQ_TIMER1,timer);
	TIMER_CR(1) = TIMER_ENABLE | TIMER_DIV_1024 | TIMER_IRQ_REQ;
	TIMER_DATA(1) = TIMER_FREQ_1024(6);
	
	constructInfoMenu();	
	//put the foreground in the front, background behind everything
	bgSetPriority(fore,0);
	bgSetPriority(back,3);
	
	videoBgDisable(fore);

}


int main(void){
	
	initialize();	
	
	irqSet(IRQ_KEYS,toggleMenu);
	REG_KEYCNT = KEY_START | KEY_SELECT | (1<<14);

	irqEnable(IRQ_KEYS);
	while(1){
		oamClear(&oamMain,0,128);		
		Game b;
		touchPosition touch;
		int lastx=-1;
		int lasty=-1;
		INDEX=2;
		Maxmin<OthelloData, BoardSig> mm;
		while(1){
			scanKeys();
			
			oamSet(&oamSub,67,96,128,0,15,opponent[0].size,opponent[0].format,opponent[INDEX].data,0,false,false,false,false,false);

			drawBoard(b);
			oamUpdate(&oamMain);
			oamUpdate(&oamSub);
			swiWaitForVBlank();
			if(b.endGame())
				break;
			if(b.turn==b.b.W){ //computer turn
				INDEX=0;
				oamSet(&oamSub,67,96,128,0,15,opponent[0].size,opponent[0].format,opponent[INDEX].data,0,false,false,false,false,false);
				oamUpdate(&oamSub);
				irqEnable(IRQ_TIMER1); //enable interupt to update screen while processing
				OthelloData in(b.b,b.turn,OthelloMove());
				OthelloData out(in);
				mm.evaluate(in,out,5);
				OthelloMove &om=out.lastMove;
				b.boardPlay(om.x,om.y);
				irqDisable(IRQ_TIMER1); //done processing, no more interupt
				INDEX=2;
				continue;			
			}
			if( !message ){
				touchRead(&touch);
				if( keysCurrent() & KEY_TOUCH ){
					oamSet(&oamMain,65,touch.px-11,touch.py-11,0,15,gray.size,gray.format,gray.data,0,false,false,false,false,false);
					lastx=touch.px;
					lasty=touch.py;
				} else {
					if(lastx!=-1){
						b.play(lastx,lasty);
						oamClear(&oamMain,65,1);
						lastx=-1;
						lasty=-1;
					}
				}
			}			
		}
		//show winning/losing faces....should be hidden away
		if( b.score(b.b.B)<b.score(b.b.W) )
			INDEX=2;
		else
			INDEX=3;
		oamSet(&oamSub,67,96,128,0,15,opponent[0].size,opponent[0].format,opponent[INDEX].data,0,false,false,false,false,false);
		oamUpdate(&oamSub);
				
		// wait for touch to restart
		do{
			swiWaitForVBlank();
			scanKeys();
			touchRead(&touch);
		} while( !(keysCurrent() & KEY_TOUCH) );
	}	
	clearSprites();
	return 0;
}
